#include <linux/config.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/security.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/devfs_fs_kernel.h>

#include "task_security.h"
#include "task_contexts.h"
#include "sec_ops.h"
#include "chrdev.h"

//#if defined(CONFIG_SECURITY_PCONTEXT_MODULE)
#define MY_NAME THIS_MODULE->name
//#else
//#define MY_NAME "tcontext"
//#endif

static int is_secondary = 0;
static int cleanup_on_exit = 0;

void my_unreg_security(void)
{
	if (is_secondary) {
		if (mod_unreg_security(MY_NAME, &tcontext_sec_ops)) {
			printk(KERN_ERR "failure unregistering tcontext module with primary module\n");
		}
	} else {
		if (unregister_security(&tcontext_sec_ops)) {
			printk(KERN_ERR "failure unregistering tcontext module with kernel\n");
		}
	}
}

void tcontext_module_exit(void)
{
	struct task_struct *p;
	
	tcontext_chrdev_unregister();
	
	my_unreg_security();
	
	kobject_unregister(&root_task_context->kobj);

	/* cleanup any remaining task_security_t structures */
	if (cleanup_on_exit) {
		spin_lock(&tasklist_lock);
		for_each_process(p) {
			if (!task_security(p))
				continue;
			free_task_security(p);
		}
		spin_unlock(&tasklist_lock);
	}
}


int tcontext_module_init(void)
{
	int err = -ENOSYS;

	err = create_root_task_context();
	if (err)
		goto out;
	
	is_secondary = 0;
	if (register_security(&tcontext_sec_ops)) {
		if (mod_reg_security(MY_NAME, &tcontext_sec_ops)) {
			printk (KERN_ERR "Failure registering tcontext module with the kernel or primary security module.\n");
			return -EINVAL;
		}
		is_secondary = 1;
	}
	
	err = tcontext_chrdev_register();
	if (err) {
		goto out1;
	}
	
	return 0;
out1:
	my_unreg_security();

out:
	if (root_task_context)
		kobject_unregister(&root_task_context->kobj);
	return err;
}

module_init(tcontext_module_init);
module_exit(tcontext_module_exit);

MODULE_AUTHOR ("Dwayne C. Litzenberger");
MODULE_DESCRIPTION ("Task Contexts");
MODULE_LICENSE ("GPL");

MODULE_PARM(ptrace_mode, "i");
MODULE_PARM_DESC(ptrace_mode, "0 => normal, 1 => disabled, 2 => root context only");

MODULE_PARM(cleanup_on_exit, "i");
MODULE_PARM_DESC(cleanup_on_exit, "0 => normal, 1=> free all tsk->security on rmmod");
