#ifndef TASK_CONTEXTS_H
#define TASK_CONTEXTS_H

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kobject.h>
#include <linux/sched.h>

#define task_context(tsk) (task_security(tsk) ? \
		(task_security(tsk)->context) : NULL)

typedef struct {
	struct kobject kobj;
} task_context_t;

extern task_context_t *root_task_context;
extern int create_root_task_context(void);

/*
 * @create_task_context:
 * 	Create a new task context.
 * 	@parent parent task context, or NULL.
 * 	@format printf-style format string
 *
 * @is_task_subcontext:
 * 	Check if @child is a sub-context of @parent.
 * 	@child points to the child context
 * 	@parent points to the parent context
 * 	Return 1 if @child is a sub-context of @parent, otherwise, return 0.
 */


extern task_context_t *create_task_context(task_context_t *parent);

extern int is_task_subcontext(task_context_t *child, task_context_t *parent);

static inline task_context_t *task_context_parent(task_context_t *tc)
{
	return container_of(tc->kobj.parent, task_context_t, kobj);
}

extern int attach_task_context(struct task_struct *tsk, task_context_t *tc);
extern void detach_task_context(struct task_struct *tsk);


#endif /* TASK_CONTEXTS_H */
